/***************************************************************************
 *   Copyright (C) 2006 by Ken Werner                                      *
 *   ken.werner@web.de                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef SOURCE_H
#define SOURCE_H
#include <qstring.h>

#include "sourceprefs.h"
#include <kconfig.h>

// Forward Declarationss
class QVBoxLayout;

/**
 * This abstract Source is the super class of all sources and provides just the basics.
 * @author Ken Werner
 */
class Source : public QObject {
	Q_OBJECT //macro which activates signals and slots (moc)
public:
	/**
	 * Creates a new Source
	 */
	Source(QWidget* inParent);
	virtual ~Source();

	/**
	 * Returns the internal Name of this source
	 */
	const QString& getID() const;

	/**
	 * Returns the Name of this source
	 */
	const QString& getName() const;

	/**
	 * Returns the Description of this source
	 */
	const QString& getDescription() const;

	/**
	 * Returns true if this source is currently enabled otherwise false
	 */
	bool isEnabled() const;

	/**
	 * Returns true if this source should show it's name in kicker otherwise false
	 */
	bool showName() const;

	/**
	 * Returns true if this source is currently enabled otherwise false
	 */
	bool isToolTipEnabled() const;

	/**
	 * Returns the widget of this source that is displayed in the kicker
	 */
	virtual QWidget* getWidget() = 0;

	/**
	 * Returns the value of this source
	 * This is used to display the ToolTip text
	 */
	virtual QString getValue() const = 0;

	/**
	 * Creates the preference panel of this source
	 */
	virtual QWidget* createPrefs(QWidget* inParent);

	/**
	 * returnes the preference panel of this source
	 */
	virtual SourcePrefs* getPrefs();

	/**
	 * fills the prefs gui with appropriate values
	 */
	virtual void updatePrefsGUI();

public slots:
	/**
	 * Sets whether this source is enabled or not
	 */
	void setEnabled(bool inEnabled);

	/**
	 * Applies the prefs
	 */
	virtual void applyPrefs();

	/**
	 * Saves the prefs (implicit apply)
	 */
	virtual void savePrefs(KConfig* inKConfig);

	/**
	 * Loads the prefs
	 */
	virtual void loadPrefs(KConfig* inKConfig);

signals:
	/**
	 * This signal is emitted whenever the enabled flag (mEnabled) of this source has changed
	 */
	void enabledChanged(bool inEnabled, QWidget* inWidget);

protected:
	/**
	 * This method can be overridden in sub classes to add specific the preference panels
	 */
	virtual QWidget* addPrefs(QWidget* inParent);

	/** 
	 * The ID of the source
	 * must be unique among the sources 
	 */
	QString mID; 
	/** 
	 * The name of that source showed in the kicker
	 */
	QString mName;
	/** 
	 * The description of that source
	 */
	QString mDescription;
	/** 
	 * Indicates whether that source is enabled (showed on the kicker) or not
	 */
	bool mEnabled;
	/** 
	 * Indicates whether to show the name in kicker or not
	 */
	bool mShowName;
	/** 
	 * Indicates whether that source is showed on the tooltip or not
	 */
	bool mToolTipEnabled;

private:
	/** 
	 * The preference panel of that source
	 */
	SourcePrefs* mSourcePrefs;
};
#endif //SOURCE_H
